'' =================================================================================================
''
''   File....... qs_vixen_streamer.spin
''   Purpose.... Receives Vixen stream, converts to LED brightness
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2011-2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com 
''   Started.... 
''   Updated.... 15 JUL 2012
''
'' =================================================================================================

{{

  Connect Vixen to QuickStart or Demo Board using normal programming connnection
  -- use Generic Serial output driver
  -- set baud to 115.2K
  -- no header or footer

}}


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LED8 = 23
  LED7 = 22                                                     ' QuickStart / Demo Board LEDs
  LED6 = 21   
  LED5 = 20 
  LED4 = 19
  LED3 = 18
  LED2 = 17   
  LED1 = 16 


con

  CHANNELS = 8
  BAUD     = 115_200                                            ' from Vixen

  
obj

  serial : "fullduplexserial64"                                 ' serial input (from Vixen)
  leds   : "jm_pwm8"                                            ' pwm driver for dimmer outputs
      

var


pub main | level, ch

  serial.start(RX1, TX1, %0000, BAUD)                           ' start serial input
  leds.start(8, LED1)                                           ' start pwm for outputs

  pause(10)                                                     ' let things settle down
  serial.rxflush                                                ' clean-up noise

  repeat
    repeat
      level := serial.rxtime(3)                                 ' wait for 3ms idle period
    until (level < 0)

    repeat                                                      ' wait for start of frame
      level := serial.rxcheck
    until (level => 0)                                          
    leds.set(0, level)

    repeat ch from 1 to CHANNELS-1                              ' rx rest of frame
      level := serial.rxtime(1)  
      leds.set(ch, level)
      

con

  { ------------- }
  {  B A S I C S  } 
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}